#include <gammu.h>

extern void GetAllNotes(int argc, char *argv[]);
extern void GetAllToDo(int argc, char *argv[]);
extern void GetToDo(int argc, char *argv[]);
extern void ListToDoCategory(int argc, char *argv[]);
extern void PrintToDo(GSM_ToDoEntry * ToDo);
extern void DeleteToDo(int argc, char *argv[]);
extern void GetCalendarSettings(int argc, char *argv[]);
extern void GetAllCalendar(int argc, char *argv[]);
extern void GetCalendar(int argc, char *argv[]);
extern void SetAlarm(int argc, char *argv[]);
extern void GetAlarm(int argc, char *argv[]);
extern void SetDateTime(int argc, char *argv[]);
extern void GetDateTime(int argc, char *argv[]);
extern void PrintCalendar(GSM_CalendarEntry * Note);
extern void DeleteCalendar(int argc, char *argv[]);
