/* Test for line splitting code */

#include <gammu.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "../libgammu/misc/misc.h"

#define LONGLINE "" \
"1 - The American nation in the sixth ward is a fine people; they love theeagle -- on the back of a dollar.		-- Finlay Peter Dunne" \
"2 - Q:	What's the difference between the 1950's and the 1980's?A:	In the 80's, a man walks into a drugstore and states loudly, \"I'd	like some condoms,\" and then, leaning over the counter, whispers,	\"and some cigarettes.\"" \
"3 - F:	When into a room I plunge, I	Sometimes find some VIOLET FUNGI.	Then I linger, darkly brooding	On the poison they're exuding.		-- The Roguelet's ABC" \
"4 - ... a thing called Ethics, whose nature was confusing but if you had it youwere a High-Class Realtor and if you hadn't you were a shyster, a piker anda fly-by-night.  These virtues awakened Confidence and enabled you to handleBigger Propositions.  But they didn't imply that you were to be impracticaland refuse to take twice the value for a house if a buyer was such an idiotthat he didn't force you down on the asking price.		-- Sinclair Lewis, \"Babbitt\"" \
"5 - Nature to all things fixed the limits fit,And wisely curbed proud man's pretending wit.As on the land while here the ocean gains,In other parts it leaves wide sandy plains;Thus in the soul while memory prevails,The solid power of understanding fails;Where beams of warm imagination play,The memory's soft figures melt away.		-- Alexander Pope (on runtime bounds checking?)" \
"6 - lp1 on fire(One of the more obfuscated kernel messages)" \
"7 - C is quirky, flawed, and an enormous success		-- Dennis M. Ritchie" \
"8 - Laugh and the world laughs with you, snore and you sleep alone." \
"9 - Q:	How many Martians does it take to screw in a light bulb?A:	One and a half." \
"10 - Fear and loathing, my man, fear and loathing.		-- H. S. Thompson"

#define LASTLINE "LAST"

const char input[] =
    LONGLINE "\n"
    "1 - Wrinkles should merely indicate where smiles have been.\n"
    "2 - <marcus> dunham: You know how real numbers are constructed from rational\n"
    "3 - Truth is free, but information costs.\n"
    "4 - <Knghtbrd> Yorick: no problem with indexed color palettes for images, as\n"
    "5 - A violent man will die a violent death.\n"
    "6 - The inclusion of any link does not imply endorsement of the site.\n"
    "7 - Comparing information and knowledge is like asking whether the fatness\n"
    "8 - When taxes are due, Americans tend to feel quite bled-white and blue.\n"
    "9 - As to Jesus of Nazareth...I think the system of Morals and his Religion,\n"
    "10 - TOO BAD YOU CAN'T BUY a voodoo globe so that you could make the earth spin\n"
    "11 - When you are in it up to your ears, keep your mouth shut.\n"
    "12 - Just because he's dead is no reason to lay off work.\n"
    "13 - People tend to make rules for others and exceptions for themselves.\n"
    "14 - The difference between art and science is that science is what we\n"
    "15 - \"...I could accept this openness, glasnost, perestroika, or whatever you want\n"
    "16 - It is annoying to be honest to no purpose.\n"
    "17 - ... we must not judge the society of the future by considering whether or not\n"
    "18 - \"Everybody is talking about the weather but nobody does anything about it.\"\n"
    "19 - QOTD:\n"
    "20 - Fascinating is a word I use for the unexpected.\n"
    "21 - Go away, I'm all right.\n"
    "22 - History shows that the human mind, fed by constant accessions of knowledge,\n"
    "23 - People who think they know everything greatly annoy those of us who do.\n"
    "24 - Small is beautiful.\n"
    "25 - \"His eyes were cold.  As cold as the bitter winter snow that was falling\n"
    "26 - Due to circumstances beyond your control, you are master of your fate\n"
    "27 - algorithm, n.:\n"
    "28 - PL/I -- \"the fatal disease\" -- belongs more to the problem set than to the\n"
    "29 - This sad little lizard told me that he was a brontosaurus on his mother's\n"
    "30 - \"Elves and Dragons!\" I says to him.  \"Cabbages and potatoes are better\n"
    "31 - * Mercury calmly removes XT-Ream's arm..\n"
    "32 - Beggars should be no choosers.\n"
    "33 - Every man who has reached even his intellectual teens begins to suspect\n"
    "34 - You possess a mind not merely twisted, but actually sprained.\n"
    "35 - Man invented language to satisfy his deep need to complain.\n"
    "36 - grep me no patterns and I'll tell you no lines.\n"
    "37 - Psychologists think they're experimental psychologists.\n"
    "38 - God made everything out of nothing, but the nothingness shows through.\n"
    "39 -   The first condition of immortality is death. -Stanislaw Lec\n"
    "40 - FORTUNE'S GUIDE TO DEALING WITH REAL-LIFE SCIENCE FICTION: #6\n"
    "41 - When you are working hard, get up and retch every so often.\n"
    "42 - An editor is one who separates the wheat from the chaff and prints the chaff.\n"
    "43 - The cost of living hasn't affected its popularity.\n"
    "44 - We're Knights of the Round Table\n"
    "45 - \"Who cares if it doesn't do anything?  It was made with our new\n"
    "46 - Here I am in the POSTERIOR OLFACTORY LOBULE but I don't see CARL SAGAN\n"
    "47 - How many Zen Buddhist does it take to change a light bulb?\n"
    "48 - I'm not prejudiced, I hate everyone equally.\n"
    "49 - Most people don't need a great deal of love nearly so much as they need\n"
    "50 - The aim of science is to seek the simplest explanations of complex\n"
    "51 - QOTD:\n"
    "52 -   I never liked you, and I always will. -Samuel Goldwyn\n"
    "53 - Mr and Mrs PED, can I borrow 26.7% of the RAYON TEXTILE production of\n"
    "54 - <toor> netgod: what do you have in your kernel??? The compiled source for\n"
    "55 - Every successful person has had failures but repeated failure is no\n"
    "56 - Vests are to suits as seat-belts are to cars.\n"
    "57 - Keep grandma off the streets -- legalize bingo.\n"
    "58 - 	A ranger was walking through the forest and encountered a hunter\n"
    "59 - All of a sudden, I want to THROW OVER my promising ACTING CAREER, grow\n"
    "60 - He who has the courage to laugh is almost as much a master of the world\n"
    "61 - > I'm an idiot..  At least this [bug] took about 5 minutes to find..\n"
    "62 - Many years ago in a period commonly know as Next Friday Afternoon,\n"
    "63 - Computers can figure out all kinds of problems, except the things in\n"
    "64 - The street preacher looked so baffled\n"
    "65 - Will Rogers never met you.\n"
    "66 - It's not whether you win or lose but how you played the game.\n"
    "67 - \"Let us condemn to hellfire all those who disagree with us.\"\n"
    "68 - Lack of skill dictates economy of style.\n"
    "69 - If you are going to walk on thin ice, you may as well dance.\n"
    "70 - RELATIVES!!\n"
    "71 - Professional sample - not for sale.\n"
    "72 - The real man's Bloody Mary:\n"
    "73 - 	\"A penny for your thoughts?\"\n"
    "74 - If everything seems to be going well, you have obviously overlooked something.\n"
    "75 - Although written many years ago, Lady Chatterley's Lover has just been\n"
    "76 - Down-Nesting:\n"
    "77 - Truly simple systems... require infinite testing.\n"
    "78 - \"Eat, drink, and be merry, for tomorrow you may work.\"\n"
    "79 - There's only one everything.\n"
    "80 - 	\"Do you think there's a God?\"\n"
    "81 - I have already given two cousins to the war and I stand ready to sacrifice\n"
    "82 - Be security conscious -- National defense is at stake.\n"
    "83 - First study the enemy.  Seek weakness.\n"
    "84 - Dear Emily, what about test messages?\n"
    "85 - Families, when a child is born\n"
    "86 - Protect from light.\n"
    "87 - \"I changed my headlights the other day. I put in strobe lights instead! Now\n"
    "88 - May you have many beautiful and obedient daughters.\n"
    "89 - Moderation in all things.\n"
    "90 - War is much too serious a matter to be entrusted to the military.\n"
    "91 - A violent man will die a violent death.\n"
    "92 - \"We can't schedule an orgy, it might be construed as fighting\"\n"
    "93 - ignisecond, n:\n"
    "94 - A tautology is a thing which is tautological.\n"
    "95 - Management is not responsible for loss or damage.\n"
    "96 - If you keep anything long enough, you can throw it away.\n"
    "97 - Thus spake the master programmer:\n"
    "98 - Once, adv.:\n"
    "99 - Fascinating is a word I use for the unexpected.\n"
    "100 - It took me fifteen years to discover that I had no talent for writing,\n"
    "101 - Perhaps the biggest disappointments were the ones you expected anyway.\n"
    "102 - A man paints with his brains and not with his hands.\n"
    "103 - Major premise:\n"
    "104 - Hier liegt ein Mann ganz obnegleich;\n"
    "105 - If any man wishes to be humbled and mortified, let him become president\n"
    "106 - <Manoj> I *like* the chicken\n"
    "107 - The man she had was kind and clean\n"
    "108 - The brain is a wonderful organ; it starts working the moment you get up\n"
    "109 - A woman forgives the audacity of which her beauty has prompted us to be guilty.\n"
    "110 - HOW YOU CAN TELL THAT IT'S GOING TO BE A ROTTEN DAY:\n"
    "111 - Question authority.\n"
    "112 - \"Gentlemen of the jury,\" said the defense attorney, now beginning\n"
    "113 - \"One size fits all\":\n"
    "114 - Truth is the most valuable thing we have -- so let us economize it.\n"
    "115 - In every job that must be done, there is an element of fun.\n"
    "116 - Live Free or Live in Massachusetts.\n"
    "117 - 	Something mysterious is formed, born in the silent void.  Waiting\n"
    "118 - court, n.:\n"
    "119 - Well, don't worry about it...  It's nothing.\n"
    "120 - One expresses well the love he does not feel.\n"
    "121 - <Slackware> uh oh, what have I started :)\n"
    "122 - Tax, title, tag, and dealer handling not included.\n"
    "123 - > Ok, I see you know what you're doing :-)\n"
    "124 - This TOPS OFF my partygoing experience!  Someone I DON'T LIKE is\n"
    "125 - The problem with most conspiracy theories is that they seem to believe that\n"
    "126 - I had the rare misfortune of being one of the first people to try and\n"
    "127 - The Heineken Uncertainty Principle:\n"
    "128 - Adam was but human--this explains it all.  He did not want the apple for the\n"
    "129 - Even if we put all these nagging thoughts [four embarrassing questions about\n"
    "130 - It's amazing how much better you feel once you've given up hope.\n"
    "131 - So much\n"
    "132 - I need to discuss BUY-BACK PROVISIONS with at least six studio SLEAZEBALLS!!\n"
    "133 - FORTUNE DISCUSSES THE DIFFERENCES BETWEEN MEN AND WOMEN:	#8\n"
    "134 - The all-softening overpowering knell,\n"
    "135 - Carperpetuation (kar' pur pet u a shun), n.:\n"
    "136 - It matters not whether you win or lose; what matters is whether I win or lose.\n"
    "137 - Fascinating, a totally parochial attitude.\n"
    "138 - I am convinced that the truest act of courage is to sacrifice ourselves\n"
    "139 - Q:	How many journalists does it take to screw in a light bulb?\n"
    "140 - Algol-60 surely must be regarded as the most important programming language\n"
    "141 - Even if you aren't in doubt, consider the mental welfare of the person who\n"
    "142 - Infancy, n.:\n"
    "143 - I cannot and will not cut my conscience to fit this year's fashions.\n"
    "144 - Today is a good day for information-gathering.  Read someone else's mail file.\n"
    "145 - It is now quite lawful for a Catholic woman to avoid pregnancy by a resort to\n"
    "146 - A hacker does for love what others would not do for money.\n"
    "147 - The danger is not that a particular class is unfit to govern.  Every class\n"
    "148 - As long as there are ill-defined goals, bizarre bugs, and unrealistic\n"
    "149 - Ambition is a poor excuse for not having sense enough to be lazy.\n"
    "150 - Treaties are like roses and young girls -- they last while they last.\n"
    "151 - I develop for Linux for a living, I used to develop for DOS.\n"
    "152 - I have become me without my consent.\n"
    "153 - Quick, sing me the BUDAPEST NATIONAL ANTHEM!!\n"
    "154 - Uh-oh -- WHY am I suddenly thinking of a VENERABLE religious leader\n"
    "155 - Academicians care, that's who.\n"
    "156 - <RoboHak> hmm, lunch does sound like a good idea\n"
    "157 - Just because I turn down a contract on a guy doesn't mean he isn't going\n"
    "158 - Q:	How do you catch a unique rabbit?\n"
    "159 - You can fool some of the people some of the time,\n"
    "160 - \"The History of every major Galactic Civilization tends to pass through\n"
    "161 - A modem is a baudy house.\n"
    "162 - \"That boy's about as sharp as a pound of wet liver\"\n"
    "163 - At the heart of science is an essential tension between two seemingly\n"
    "164 - QOTD:\n"
    "165 - Successophobia:\n"
    "166 - Alea iacta est.\n"
    "167 - Save gas, don't use the shell.\n"
    "168 - There was a writer in 'Life' magazine ... who claimed that rabbits have\n"
    "169 - \"Here's something to think about:  How come you never see a headline like\n"
    "170 - Don't hate yourself in the morning -- sleep till noon.\n"
    "171 - Your mode of life will be changed for the better because of new developments.\n"
    "172 - My father was a saint, I'm not.\n"
    "173 - To every Ph.D. there is an equal and opposite Ph.D.\n"
    "174 - You are dishonest, but never to the point of hurting a friend.\n"
    "175 - Dry clean only.\n"
    "176 - Teach children to be polite and courteous in the home, and, when they grow up,\n"
    "177 - Different all twisty a of in maze are you, passages little.\n"
    "178 - Nudists are people who wear one-button suits.\n"
    "179 - Check here if tax deductible.\n"
    "180 - Nobody knows what goes between his cold toes and his warm ears.\n"
    "181 - I feel like I'm in a Toilet Bowl with a thumbtack in my forehead!!\n"
    "182 - Q:	How many WASPs does it take to change a light bulb?\n"
    "183 - Don't read everything you believe.\n"
    "184 - QOTD:\n"
    "185 - You are always busy.\n"
    "186 - Puns are little \"plays on words\" that a certain breed of person loves to\n"
    "187 - If you want divine justice, die.\n"
    "188 - You roll my log, and I will roll yours.\n"
    "189 - Tempt me with a spoon!\n"
    "190 - You will be surrounded by luxury.\n"
    "191 - He who spends a storm beneath a tree, takes life with a grain of TNT.\n"
    "192 - If you just try long enough and hard enough, you can always manage to\n"
    "193 - Usually, in the studio, on this sort of thing ... you just go out and have\n"
    "194 - An honest tale speeds best being plainly told.\n"
    "195 - It's clever, but is it art?\n"
    "196 - The only people for me are the mad ones -- the ones who are mad to live,\n"
    "197 - You have an ambitious nature and may make a name for yourself.\n"
    "198 - Hoaars-Faisse Gallery presents:\n"
    "199 - The real man's Bloody Mary:\n"
    "200 - When does later become never?\n"
    "201 - It's not whether you win or lose, it's how you look playing the game.\n"
    "202 - Bill Dickey is learning me his experience.\n"
    "203 - If you can keep your head when all about you are losing theirs, then\n"
    "204 - Why won't sharks eat lawyers?   Professional courtesy.\n"
    "205 - As I was walking down the street one dark and dreary day,\n"
    "206 - Some people have a way about them that seems to say: \"If I have\n"
    "207 - \"In matrimony, to hesitate is sometimes to be saved.\"\n"
    "208 - If it happens once, it's a bug.\n"
    "209 - In the broad and final sense all institutions are educational in the\n"
    "210 - Support wildlife -- vote for an orgy.\n"
    "211 - Brain fried -- Core dumped\n"
    "212 - Mencken and Nathan's Second Law of The Average American:\n"
    "213 - like:\n"
    "214 - Worst Response To A Crisis, 1985:\n"
    "215 - If a million people say a foolish thing, it is still a foolish thing.\n"
    "216 - Brain off-line, please wait.\n"
    "217 - The average nutritional value of promises is roughly zero.\n"
    "218 - Something must be Done\n"
    "219 - FORTRAN is the language of Powerful Computers.\n"
    "220 - A single flow'r he sent me, since we met.\n"
    "221 - If you don't have a nasty obituary you probably didn't matter.\n"
    "222 - A light wife doth make a heavy husband.\n"
    "223 - You can't take damsel here now.\n"
    "224 - Heuristics are bug ridden by definition.  If they didn't have bugs,\n"
    "225 - What do you have when you have six lawyers buried up to their necks in sand?\n"
    "226 - Let others praise ancient times; I am glad I was born in these.\n"
    "227 - >>> Internal error in fortune program:\n"
    "228 - We don't really understand it, so we'll give it to the programmers.\n"
    "229 - 	A manager went to the master programmer and showed him the requirements\n"
    "230 - The difference between waltzes and disco is mostly one of volume.\n"
    "231 - Little Fly,\n"
    "232 - Slaves are generally expected to sing as well as to work ... I did not, when\n"
    "233 - Did you move a lot of KOREAN STEAK KNIVES this trip, Dingy?\n"
    "234 - Rome wasn't burnt in a day.\n"
    "235 - The powers not delegated to the United States by the Constitution, nor\n"
    "236 - May contain nuts.\n"
    "237 - This unit... must... survive.\n"
    "238 - Tan me hide when I'm dead, Fred,\n"
    "239 - <james> but, then I used an Atari, I was more likely to win the lottery in\n"
    "240 - Do not disturb.\n"
    "241 - Imagine if every Thursday your shoes exploded if you tied them the usual\n"
    "242 - No one likes us.\n"
    "243 - Now I know someone out there is going to claim, \"Well then, UNIX is intuitive,\n"
    "244 - I've heard a Jew and a Muslim argue in a Damascus cafe with less passion\n"
    "245 - Pardo's First Postulate:\n"
    "246 - What PROGRAM are they watching?\n"
    "247 - There are times when truth is stranger than fiction and lunch time is one\n"
    "248 - Kirkland, Illinois, law forbids bees to fly over the village or through\n"
    "249 - They are called computers simply because computation is the only significant\n"
    "250 - You have had a long-term stimulation relative to business.\n"
    "251 - \"Can you imagine how life could be improved if we could do away with\n"
    "252 - Politics is not the art of the possible.  It consists in choosing\n"
    "253 - Give me a fish and I will eat today.\n"
    "254 - Si jeunesse savait, si vieillesse pouvait.\n"
    "255 - Who does not love wine, women, and song,\n"
    "256 - 	In the days when Sussman was a novice Minsky once came to him as he\n"
    "257 - The difference between waltzes and disco is mostly one of volume.\n"
    "258 - In 1880 the French captured Detroit but gave it back ... they couldn't\n"
    "259 - All parts should go together without forcing.  You must remember that the parts\n"
    "260 - Has anyone realized that the purpose of the fortune cookie program is to\n"
    "261 - \"You can have my Unix system when you pry it from my cold, dead fingers.\"\n"
    "262 - When I hear a man applauded by the mob I always feel a pang of pity\n"
    "263 - This night methinks is but the daylight sick.\n"
    "264 - There are three things I always forget.  Names, faces -- the third I\n"
    "265 - Satire is what closes Saturday night.\n"
    "266 - Lookie, lookie, here comes cookie...\n"
    "267 - ... the HIGHWAY is made out of LIME JELLO and my HONDA is a barbequeued\n"
    "268 - All intelligent species own cats.\n"
    "269 - If you didn't have most of your friends, you wouldn't have most of\n"
    "270 - A little kid went up to Santa and asked him, \"Santa, you know when I'm bad\n"
    "271 - Everyone is more or less mad on one point.\n"
    "272 - One of the worst of my many faults is that I'm too critical of myself.\n"
    "273 - Quidquid latine dictum sit, altum viditur.\n"
    "274 - You will be traveling and coming into a fortune.\n"
    "275 - Dijkstra probably hates me.\n"
    "276 - I feel sorry for your brain... all alone in that great big head...\n"
    "277 - I have the power to HALT PRODUCTION on all TEENAGE SEX COMEDIES!!\n"
    "278 - The so-called \"desktop metaphor\" of today's workstations is instead an\n"
    "279 - My LESLIE GORE record is BROKEN ...\n"
    "280 - Clear the laundromat!!  This whirl-o-matic just had a nuclear meltdown!!\n"
    "281 - Ignorance must certainly be bliss or there wouldn't be so many people\n"
    "282 - <Overfiend> Joy: Hey, I'm an asshole.  Assholes emit odious gas.\n"
    "283 - Why use Windows, since there is a door?\n"
    "284 - Nothing in progression can rest on its original plan.  We may as well think of\n"
    "285 - Kansas state law requires pedestrians crossing the highways at night to\n"
    "286 - Please help keep the world clean: others may wish to use it.\n"
    "287 - #define NULL 0           /* silly thing is, we don't even use this */\n"
    "288 - Like so many Americans, she was trying to construct a life that made\n"
    "289 - Ruth made a great mistake when he gave up pitching.  Working once a week,\n"
    "290 - Neurotics build castles in the sky,\n"
    "291 - What ever happened to happily ever after?\n"
    "292 - 	It took 300 years to build and by the time it was 10% built,\n"
    "293 - Jones' Motto:\n"
    "294 - Men ought to know that from the brain and from the brain only arise our\n"
    "295 - The primary function of the design engineer is to make things\n"
    "296 - Is something VIOLENT going to happen to a GARBAGE CAN?\n"
    "297 - Yeah, there are more important things in life than money, but they won't go\n"
    "298 - Happiness adds and multiplies as we divide it with others.\n"
    "299 - One small step for man, one giant stumble for mankind.\n"
    "300 - The sky is blue so we know where to stop mowing.\n"
    "301 - Today is a good day for information-gathering.  Read someone else's mail file.\n"
    "302 - \"Can you program?\"  \"Well, I'm literate, if that's what you mean!\"\n"
    "303 - If I have to lay an egg for my country, I'll do it.\n"
    "304 - The Moving Finger writes; and, having writ,\n"
    "305 - <barneyfu> knghtbrd: crap, SDL sure makes DGA a helluva alot easier too\n"
    "306 - If all the world's economists were laid end to end, we wouldn't reach a\n"
    "307 - Leveraging always beats prototyping.\n"
    "308 - A rope lying over the top of a fence is the same length on each side.  It\n"
    "309 - \"...and the fully armed nuclear warheads, are, of course, merely a\n"
    "310 - I'm wearing PAMPERS!!\n"
    "311 - For the fashion of Minas Tirith was such that it was built on seven levels,\n"
    "312 - I feel ... JUGULAR ...\n"
    "313 - 4.2 BSD UNIX #57: Sun Jun 1 23:02:07 EDT 1986\n"
    "314 - <gecko> Hmm... I wonder what else seperates Debian from the rest of the\n"
    "315 - Whistler's Law:\n"
    "316 - RAM wasn't built in a day.\n"
    "317 - If I am elected, the concrete barriers around the WHITE HOUSE will be\n"
    "318 - For every complex problem, there is a solution that is simple, neat, and wrong.\n"
    "319 - Buck-passing usually turns out to be a boomerang.\n"
    "320 - My mother is a fish.\n"
    "321 - As soon as we started programming, we found to our surprise that it wasn't\n"
    "322 - To keep your friends treat them kindly; to kill them, treat them often.\n"
    "323 - Shut off engine before fueling.\n"
    "324 - Just don't create a file called -rf.  :-)\n"
    "325 - Being a BALD HERO is almost as FESTIVE as a TATTOOED KNOCKWURST.\n"
    "326 - Not everything worth doing is worth doing well.\n"
    "327 - Calm down, it's *only* ones and zeroes.\n"
    "328 - Delay not, Caesar.  Read it instantly.\n"
    "329 - If science were explained to the average person in a way that is accessible\n"
    "330 - <Midgar> From all the sterotypes about Aussies, I figure you guys are\n"
    "331 - \"Plan to throw one away.  You will anyway.\"\n"
    "332 - To err is human, to purr feline.\n"
    "333 - \"Pay no attention to the man behind the curtain.\"\n"
    "334 - We've tried each spinning space mote\n"
    "335 - When a child is taught ... its programmed with simple instructions --\n"
    "336 - Sodd's Second Law:\n"
    "337 - Mr. DePree also expects a \"tremendous social change\" in all workplaces.  \"When\n"
    "338 - Many people are secretly interested in life.\n"
    "339 - If I don't drive around the park,\n"
    "340 - Man belongs wherever he wants to go.\n"
    "341 - I just got out of the hospital after a speed reading accident.\n"
    "342 - It looked like something resembling white marble, which was\n"
    "343 - Let us not look back in anger or forward in fear, but around us in awareness.\n"
    "344 - They went rushing down that freeway,\n"
    "345 - 'Twas midnight, and the UNIX hacks\n"
    "346 - \"Ignorance is the soil in which belief in miracles grows.\"\n"
    "347 - Advice from an old carpenter: measure twice, saw once.\n"
    "348 - Sign here without admitting guilt.\n"
    "349 - \"Floggings will continue until morale improves.\"\n"
    "350 - 	\"Wrong,\" said Renner.\n"
    "351 - * Espy ponders an uplad queue called 'hell' so I can do dupload --to hell\n"
    "352 - All men have the right to wait in line.\n"
    "353 - You're not my type.  For that matter, you're not even my species!!!\n"
    "354 - It is up to us to produce better-quality movies.\n"
    "355 - I already have too much problem with people thinking the efficiency of\n"
    "356 - Cache:\n"
    "357 - bug, n:\n"
    "358 - I must Create a System, or be enslav'd by another Man's;\n"
    "359 - You will be married within a year, and divorced within two.\n"
    "360 - Liar, n.:\n"
    "361 - I respect faith, but doubt is what gives you an education.\n"
    "362 - Thou hast seen nothing yet.\n"
    "363 - May the bluebird of happiness twiddle your bits.\n"
    "364 - One does not thank logic.\n"
    "365 - Not affiliated with the American Red Cross.\n"
    "366 - I'm gliding over a NUCLEAR WASTE DUMP near ATLANTA, Georgia!!\n"
    "367 - Drink and dance and laugh and lie\n"
    "368 - <Mercury> Someone fix it.\n"
    "369 - Don't let people drive you crazy when you know it's in walking distance.\n"
    "370 - You buttered your bread, now lie in it.\n"
    "371 - Without coffee he could not work, or at least he could not have worked in the\n"
    "372 - The hardest thing in the world to understand is the income tax.\n"
    "373 - QOTD:\n"
    "374 - I must have slipped a disk -- my pack hurts!\n"
    "375 - Wow, I'm being shot at from both sides.  That means I *must* be right.  :-)\n"
    "376 - Computers make excellent and efficient servants, but I have no wish to\n"
    "377 - The difference between America and England is that the English think 100\n"
    "378 - Fortune's Rules for Memo Wars: #2\n"
    "379 - As part of the conversion, computer specialists rewrote 1,500 programs;\n"
    "380 - Lady, lady, should you meet\n"
    "381 - Opened for inspection.\n"
    "382 - Hear me, my chiefs, I am tired; my heart is sick and sad.  From where the\n"
    "383 - You can't cross a large chasm in two small jumps.\n"
    "384 - Oh, yeah, life goes on, long after the thrill of livin' is gone.\n"
    "385 - Keep the number of passes in a compiler to a minimum.\n"
    "386 - I used to be disgusted, now I find I'm just amused.\n"
    "387 - Absence in love is like water upon fire; a little quickens, but much\n"
    "388 - PL/I -- \"the fatal disease\" -- belongs more to the problem set than to the\n"
    "389 - Q:	What do agnostic, insomniac dyslexics do at night?\n"
    "390 - <rcw> liiwi: printk(\"CPU0 on fire\n\");\n"
    "391 - So far as we are human, what we do must be either evil or good: so far\n"
    "392 - Never argue with a fool -- people might not be able to tell the difference.\n"
    "393 - Woman on Street:	Sir, you are drunk; very, very drunk.\n"
    "394 - Maintainer's Motto:\n"
    "395 - ...One thing is that, unlike any other Western democracy that I know of,\n"
    "396 - If you have received a letter inviting you to speak at the dedication of a\n"
    "397 - This  message was brought to  you by Linux, the free  unix.\n"
    "398 - Personally, I think my choice in the mostest-superlative-computer wars has to\n"
    "399 - If in doubt, mumble.\n"
    "400 - When in this world the headlines read\n"
    "401 - \"May the forces of evil become confused on the way to your house.\"\n"
    "402 - Man must shape his tools lest they shape him.\n"
    "403 - Many people write memos to tell you they have nothing to say.\n"
    "404 - \"People should have access to the data which you have about them.  There should\n"
    "405 - Men of quality are not afraid of women for equality.\n"
    "406 - There is no snooze button on a cat who wants breakfast.\n"
    "407 - When angry, count four; when very angry, swear.\n"
    "408 - LBJ, LBJ, how many JOKES did you tell today? ? !\n"
    "409 - The Great Movie Posters:\n"
    "410 - I think that's easier to read.  Pardon me.  Less difficult to read.\n"
    "411 - You ain't learning nothing when you're talking.\n"
    "412 - A Smith & Wesson beats four aces.\n"
    "413 - I'd just as soon kiss a Wookie.\n"
    "414 - Compassion -- that's the one things no machine ever had.  Maybe it's\n"
    "415 - With/Without - and who'll deny it's what the fighting's all about?\n"
    "416 - Consensus Terrorism:\n"
    "417 - The bigger they are, the harder they hit.\n"
    "418 - Psychologists think they're experimental psychologists.\n"
    "419 - QOTD:\n"
    "420 - Kindness is a language which the deaf can hear and the blind can read.\n"
    "421 - Actually, it also looks like we should optimize (13,2,42,8,'hike') into\n"
    "422 - There's a trick to the Graceful Exit.  It begins with the vision to\n"
    "423 - I am practicing a fine point of ethics.  It is acceptable to shoot back.\n"
    "424 - Everybody needs a little love sometime; stop hacking and fall in love!\n"
    "425 - Genius doesn't work on an assembly line basis.  You can't simply say,\n"
    "426 - 	I think for the most part that the readership here uses the c-word in\n"
    "427 - Anyone who thinks UNIX is intuitive should be forced to write 5000 lines of\n"
    "428 - Now I'm having INSIPID THOUGHTS about the beatiful, round wives of\n"
    "429 - Birds are entangled by their feet and men by their tongues.\n"
    "430 - I do not believe that this generation of Americans is willing to resign itself\n"
    "431 - To live is always desirable.\n"
    "432 - Reisner's Rule of Conceptual Inertia:\n"
    "433 - FORTUNE DISCUSSES THE DIFFERENCES BETWEEN MEN AND WOMEN:	#6\n"
    "434 - QOTD:\n"
    "435 - One big pile is better than two little piles.\n"
    "436 - Prepare for tomorrow -- get ready.\n"
    "437 - Genius doesn't work on an assembly line basis.  You can't simply say,\n"
    "438 - Something's rotten in the state of Denmark.\n"
    "439 - Cancel me not -- for what then shall remain?\n"
    "440 - Be careful!  UGLY strikes 9 out of 10!\n"
    "441 - Genius, n.:\n"
    "442 - If life gives you lemons, make lemonade.\n"
    "443 - I wish I was a sex-starved manicurist found dead in the Bronx!!\n"
    "444 - Q:	How many Zen masters does it take to screw in a light bulb?\n"
    "445 - Don't kid yourself.  Little is relevant, and nothing lasts forever.\n"
    "446 - Of course you can't flap your arms and fly to the moon.  After a while you'd\n"
    "447 - There is a building with four floors.  On the first floor, there\n"
    "448 - Albrecht's Law:\n"
    "449 - <joeyh> oh my, it's a UP P III.\n"
    "450 - Status Substitution:\n"
    "451 - Think big.  Pollute the Mississippi.\n"
    "452 - You single-handedly fought your way into this hopeless mess.\n"
    "453 - No matter how much you do you never do enough.\n"
    "454 - Every cloud has a silver lining; you should have sold it, and bought titanium.\n"
    "455 - My vaseline is RUNNING...\n"
    "456 - American by birth; Texan by the grace of God.\n"
    "457 - I am getting into abstract painting.  Real abstract -- no brush, no canvas,\n"
    "458 - You have a tendency to feel you are superior to most computers.\n"
    "459 - 	An airplane pilot got engaged to two very pretty women at the same\n"
    "460 - Just weigh your own hurt against the hurt of all the others, and then\n"
    "461 - One organism, one vote.\n"
    "462 - You have a strong appeal for members of your own sex.\n"
    "463 - Reality always seems harsher in the early morning.\n"
    "464 - Satire is tragedy plus time.\n"
    "465 - I KAISER ROLL?!  What good is a Kaiser Roll without a little COLE SLAW\n"
    "466 - A fanatic is a person who can't change his mind and won't change the subject.\n"
    "467 - If this fortune didn't exist, somebody would have invented it.\n"
    "468 - Vail's Second Axiom:\n"
    "469 - Good night to spend with family, but avoid arguments with your mate's\n"
    "470 - <joeyh> netgod: er, are these 2.2.0 packages 2.0.0pre9 or do you have a\n"
    "471 - Occupational Slumming:\n"
    "472 - Computer, n.:\n"
    "473 - When neither their poverty nor their honor is touched, the majority of men\n"
    "474 - That government is best which governs least.\n"
    "475 - <lilo> it's weird, when you go on a safari to Africa to catch a lion, you\n"
    "476 - You're too beautiful to ignore.  Too much woman.\n"
    "477 - Smoking Prohibited.  Absolutely no ifs, ands, or butts.\n"
    "478 - Any member introducing a dog into the Society's premises shall be\n"
    "479 - Paranoia is heightened awareness.\n"
    "480 - While the year 2000 (y2k) problem is not an issue for us, all Linux\n"
    "481 - * Culus fears perl - the language with optional errors\n"
    "482 - It's simply unbelievable how much energy and creativity people have\n"
    "483 - \"Tell the truth and run.\"\n"
    "484 - A chicken is an egg's way of producing more eggs.\n"
    "485 - It is said an Eastern monarch once charged his wise men to invent him a\n"
    "486 - Some parts of the past must be preserved, and some of the future prevented\n"
    "487 - Bypasses are devices that allow some people to dash from point A to\n"
    "488 -   I can resist everything except temptation. -Oscar Wilde\n"
    "489 - If you will practice being fictional for a while, you will understand that\n"
    "490 - Snow White has become a camera buff.  She spends hours and hours\n"
    "491 - Must be over 18.\n"
    "492 - You will get what you deserve.\n"
    "493 - Computers are not intelligent.  They only think they are.\n"
    "494 - Prediction is very difficult, especially of the future.\n"
    "495 - \"It's a dog-eat-dog world out there, and I'm wearing Milkbone underware.\"\n"
    "496 - How should I know if it works?  That's what beta testers are for.  I\n"
    "497 - Drawing on my fine command of language, I said nothing.\n"
    "498 - It took me fifteen years to discover that I had no talent for writing,\n"
    "499 - Real Users never use the Help key.\n"
    "500 - \"A mind is a terrible thing to have leaking out your ears.\"\n"
    "501 - Inglish Spocken Hier: some mangled translations\n"
    "502 - Thus spake the master programmer:\n"
    "503 - Be valiant, but not too venturous.\n"
    "504 - A conclusion is simply the place where someone got tired of thinking.\n"
    "505 - People think my friend George is weird because he wears sideburns...behind his\n"
    "506 - Lay off the muses, it's a very tough dollar.\n"
    "507 - * Phaedrus wishes he could get a machine that consists of Sparc IO,\n"
    "508 - $100 invested at 7% interest for 100 years will become $100,000, at\n"
    "509 - I love dogs, but I hate Chihuahuas.  A Chihuahua isn't a dog.  It's a rat\n"
    "510 - 	\"Welcome back for you 13th consecutive week, Evelyn.  Evelyn, will\n"
    "511 - My mother wants grandchildren, so I said, \"Mom, go for it!\"\n"
    "512 - I think irc isn't going to work though---we're running out of topic space!\n"
    "513 - Let me assure you that to us here at First National, you're not just a\n"
    "514 - [In 'Doctor' mode], I spent a good ten minutes telling Emacs what I\n"
    "515 - Every program is a part of some other program, and rarely fits.\n"
    "516 - Crito, I owe a cock to Asclepius; will you remember to pay the debt?\n"
    "517 - Cautious, careful people always casting about to preserve their\n"
    "518 - For 20 dollars, I'll give you a good fortune next time ...\n"
    "519 - Fortune suggests uses for YOUR favorite UNIX commands!\n"
    "520 - I THINK MAN INVENTED THE CAR by instinct.\n"
    "521 - Kington's Law of Perforation:\n"
    "522 - High Priest:	Armaments Chapter One, verses nine through twenty-seven:\n"
    "523 - You may worry about your hair-do today, but tomorrow much peanut butter will\n"
    "524 - There you go man,\n"
    "525 - Truly great madness can not be achieved without significant intelligence.\n"
    "526 - To iterate is human, to recurse, divine.\n"
    "527 - \"Engineering without management is art.\"\n"
    "528 - Decision maker, n.:\n"
    "529 - > You know you are \"there\" when you are known by your first name, and\n"
    "530 - The net is like a vast sea of lutefisk with tiny dinosaur brains embedded\n"
    "531 - \"slackware users don't matter. in my experience, slackware users are\n"
    "532 - Q:	How many college football players does it take to screw in a light bulb?\n"
    "533 - Better tried by twelve than carried by six.\n"
    "534 - Have a nice diurnal anomaly.\n"
    "535 - I am firm.  You are obstinate.  He is a pig-headed fool.\n"
    "536 -   I must follow the people. Am I not their leader? -Benjamin Disraeli\n"
    "537 - Results are not typical.\n"
    "538 - Human beings were created by water to transport it uphill.\n"
    "539 - Perl is designed to give you several ways to do anything, so\n"
    "540 - God doesn't play dice.\n"
    "541 - I respect the institution of marriage.  I have always thought that every\n"
    "542 - Next to being shot at and missed, nothing is really quite as satisfying\n"
    "543 - I must get out of these wet clothes and into a dry Martini.\n"
    "544 - His designs were strictly honourable, as the phrase is: that is, to rob\n"
    "545 - I cannot believe that God plays dice with the cosmos.\n"
    "546 - From 0 to \"what seems to be the problem officer\" in 8.3 seconds.\n"
    "547 - No passes accepted for this engagement.\n"
    "548 - semper en excretus\n"
    "549 - The computer can't tell you the emotional story.  It can give you the exact\n"
    "550 - Impartial, adj.:\n"
    "551 - Time to be aggressive.  Go after a tattooed Virgo.\n"
    "552 - Diplomacy is about surviving until the next century.  Politics is about\n"
    "553 - The joys of love made her human and the agonies of love destroyed her.\n"
    "554 - It's just a jump to the left\n"
    "555 - Best if used before date on carton.\n"
    "556 - Hear me, my chiefs, I am tired; my heart is sick and sad.  From where the\n"
    "557 - USENET would be a better laboratory if there were more labor and less oratory.\n"
    "558 - As for the basic assumptions about individuality and self, this is the core\n"
    "559 - \"You can't teach seven foot.\"\n"
    "560 - 'Course, I haven't weighed in yet.  :-)\n"
    "561 - History repeats itself -- the first time as a tragi-comedy, the second\n"
    "562 - \"We cannot put off living until we are ready.  The most salient characteristic\n"
    "563 - The use of anthropomorphic terminology when dealing with computing systems\n"
    "564 - Oh, when I was in love with you,\n"
    "565 - Only the fittest survive. The vanquished acknowledge their unworthiness by\n"
    "566 - Your mode of life will be changed to EBCDIC.\n"
    "567 - I suppose some of the variation between Boston drivers and the rest of the\n"
    "568 - I just need enough to tide me over until I need more.\n"
    "569 - I am myself plus my circumstance, and if I do not save it, I cannot\n"
    "570 - Liberty is always dangerous, but it is the safest thing we have.\n"
    "571 - One size fits all.\n"
    "572 - QOTD:\n"
    "573 - You know you're in trouble when...\n"
    "574 - Change your thoughts and you change your world.\n"
    "575 - \"He didn't run for reelection.	`Politics brings you into contact with all the\n"
    "576 - It is not good for a man to be without knowledge,\n"
    "577 - One is not superior merely because one sees the world as odious.\n"
    "578 - Needs are a function of what other people have.\n"
    "579 - Now I'm being INVOLUNTARILY shuffled closer to the CLAM DIP with the\n"
    "580 - One toke over the line, sweet Mary,\n"
    "581 - You're using a keyboard!  How quaint!\n"
    "582 - This will be a memorable month -- no matter how hard you try to forget it.\n"
    "583 - The sheep died in the wool.\n"
    "584 - Some stirring may be necessary to achieve proper consistency.\n"
    "585 - Mater artium necessitas.\n"
    "586 - Egotism, n:\n"
    "587 - She's learned to say things with her eyes that others waste time putting\n"
    "588 - She cried, and the judge wiped her tears with my checkbook.\n"
    "589 - It's pretty hard to tell what does bring happiness; poverty and wealth\n"
    "590 - No animal should ever jump on the dining room furniture unless\n"
    "591 - The ladies men admire, I've heard,\n"
    "592 - Is it possible that software is not like anything else, that it is meant to\n"
    "593 - The only \"ism\" Hollywood believes in is plagiarism.\n"
    "594 - In the future, you're going to get computers as prizes in breakfast cereals.\n"
    "595 - There are two ways of disliking art.   One is to dislike it.  The other is\n"
    "596 - This life is a test.  It is only a test.  Had this been an actual life, you\n"
    "597 - Thus spake the master programmer:\n"
    "598 -  All generalisations are dangerous, including this one.\n"
    "599 - Excitement and danger await your induction to tracer duty!  As a tracer,\n"
    "600 - Many people feel that they deserve some kind of recognition for all the\n"
    "601 - But soft you, the fair Ophelia:\n"
    "602 - Subject: Bug#42432: debian-policy: Proposal for CTV for Draft for Proof of\n"
    "603 - You will probably marry after a very brief courtship.\n"
    "604 - Few things are harder to put up with than the annoyance of a good example.\n"
    "605 - Once upon a time there was a kingdom ruled by a great bear.  The peasants\n"
    "606 - Compliment, n.:\n"
    "607 - In 1962, you could buy a pair of SHARKSKIN SLACKS, with a \"Continental\n"
    "608 - \"The porcupine with the sharpest quills gets stuck on a tree more often.\"\n"
    "609 - Established technology tends to persist in the face of new technology.\n"
    "610 - All programmers are playwrights and all computers are lousy actors.\n"
    "611 - The primary difference [...] is that the Java programm will reliably and\n"
    "612 - Alan E. Davis: Some files at llug.sep.bnl.gov/pub/debian/Incoming are\n"
    "613 - FORTRAN rots the brain.\n"
    "614 - ...computer hardware progress is so fast.  No other technology since\n"
    "615 - All that is gold does not glitter,\n"
    "616 - \"I'd love to go out with you, but I'm attending the opening of my garage door.\"\n"
    "617 - You can't break eggs without making an omelet.\n"
    "618 - Fashion is a form of ugliness so intolerable that we have to alter it\n"
    "619 - Man is a rational animal who always loses his temper when he is called upon\n"
    "620 - Sometime in 1993 NANCY SINATRA will lead a BLOODLESS COUP on GUAM!!\n"
    "621 - \"They that can give up essential liberty to obtain a little temporary\n"
    "622 - It is now 10 p.m.  Do you know where Henry Kissinger is?\n"
    "623 - <aph> manoj is going nuts on the bug fixing crusade!  woo woo!\n"
    "624 - It appears that PL/I (and its dialects) is, or will be, the most widely\n"
    "625 - Good advice is something a man gives when he is too old to set a bad\n"
    "626 - Let me assure you that to us here at First National, you're not just a\n"
    "627 - We promise according to our hopes, and perform according to our fears.\n"
    "628 - You should emulate your heros, but don't carry it too far.  Especially\n"
    "629 - Japanese Minimalism:\n"
    "630 - FORTRAN is not a flower but a weed -- it is hardy, occasionally blooms,\n"
    "631 - I BET WHAT HAPPENED was they discovered fire and invented the wheel on\n"
    "632 - A shortcut is the longest distance between two points.\n"
    "633 - An apple a day makes 365 apples a year.\n"
    "634 - I'm receiving a coded message from EUBIE BLAKE!!\n"
    "635 - Genuine happiness is when a wife sees a double chin on her husband's\n"
    "636 - Eeny, Meeny, Jelly Beanie, the spirits are about to speak!\n"
    "637 - > I've hacked the Xaw3d library to give you a Win95 like interface and it\n"
    "638 - A man may sometimes be forgiven the kiss to which he is not entitled,\n"
    "639 - \"You stay here, Audrey -- this is between me and the vegetable!\"\n"
    "640 - Must be getting close to town -- we're hitting more people.\n"
    "641 - The morning sun when it's in your face really shows your age,\n"
    "642 - 	\"The pyramid is opening!\"\n"
    "643 - As I argued in \"Beloved Son\", a book about my son Brian and the subject\n"
    "644 - Sometimes, too long is too long.\n"
    "645 - <dark> \"Let's form the Linux Standard Linux Standardization Association\n"
    "646 - I'd rather have two girls at 21 each than one girl at 42.\n"
    "647 - A synonym is a word you use when you can't spell the word you first\n"
    "648 - Conceptual integrity in turn dictates that the design must proceed\n"
    "649 - We prefer to speak evil of ourselves rather than not speak of ourselves at all.\n"
    "650 - If someone had told me I would be Pope one day, I would have studied harder.\n"
    "651 - Simplicity does not precede complexity, but follows it.\n"
    "652 - Male, n.:\n"
    "653 - * Knghtbrd unleashes a pair of double barreled snurf guns and covers\n"
    "654 - H. L. Mencken's Law:\n"
    "655 - Virginia law forbids bathtubs in the house; tubs must be kept in the yard.\n"
    "656 - Odd that we think definitions are definitive.   :-)\n"
    "657 - Save the whales.  Collect the whole set.\n"
    "658 - <joeyh> oh my, it's a UP P III.\n"
    "659 - All seems condemned in the long run to approximate a state akin to\n"
    "660 - Well, the handwriting is on the floor.\n"
    "661 - Get forgiveness now -- tomorrow you may no longer feel guilty.\n"
    "662 - At participating locations only.\n"
    "663 - This  message was brought to  you by Linux, the free  unix.\n"
    "664 - Default, n.:\n"
    "665 - The greatest love is a mother's, then a dog's, then a sweetheart's.\n"
    "666 - system-independent, adj.:\n"
    "667 - Better to use medicines at the outset than at the last moment.\n"
    "668 - To err is human, to purr feline.\n"
    "669 - (1)	Office employees will daily sweep the floors, dust the\n"
    "670 - I have a simple philosophy:\n"
    "671 - Boren's Laws:\n"
    "672 - So many women; so little time!\n"
    "673 - Envy is a pain of mind that successful men cause their neighbors.\n"
    "674 - No man is an island, but some of us are long peninsulas.\n"
    "675 - 	A ranger was walking through the forest and encountered a hunter\n"
    "676 - ... I want a COLOR T.V. and a VIBRATING BED!!!\n"
    "677 - Hardware, n.:\n"
    "678 - Life is like arriving late for a movie, having to figure out what was\n"
    "679 - \"I don't mind going nowhere as long as it's an interesting path.\"\n"
    "680 - I used to think I was a child; now I think I am an adult -- not because\n"
    "681 - If you want me to be a good little bunny just dangle some carats in front\n"
    "682 - The University of California Statistics Department; where mean is normal,\n"
    "683 - Architectural Indigestion:\n"
    "684 - Demand the establishment of the government in its rightful home at Disneyland.\n"
    "685 - \"... the Mayo Clinic, named after its founder, Dr. Ted Clinic ...\"\n"
    "686 - While walking down a crowded\n"
    "687 - All who joy would win Must share it --\n"
    "688 - <Knghtbrd> Granted, RMS is a fanatic, I don't deny this.  I'll even say\n"
    "689 - Now is the time for all good men to come to.\n"
    "690 - Eh, that's it, I guess.  No 300 million dollar unveiling event for this\n"
    "691 - And on the eighth day, we bulldozed it.\n"
    "692 - VMS, n.:\n"
    "693 - \"Neighbors!!  We got neighbors!  We ain't supposed to have any neighbors, and\n"
    "694 - If you see an onion ring -- answer it!\n"
    "695 - Awright, which one of you hid my PENIS ENVY?\n"
    "696 - You can get much further with a kind word and a gun than you can with a\n"
    "697 - 	\"Somewhere\", said Father Vittorini, \"did Blake not speak of the\n"
    "698 - 	My dear People.\n"
    "699 - Fortune favors the lucky.\n"
    "700 - Take what you can use and let the rest go by.\n"
    "701 - [He] took me into his library and showed me his books, of which he had\n"
    "702 - Reply hazy, ask again later.\n"
    "703 - 	A reader reports that when the patient died, the attending doctor\n"
    "704 - Words are the voice of the heart.\n"
    "705 - Thus spake the master programmer:\n"
    "706 - 	Youth is not a time of life, it is a state of mind; it is a temper of\n"
    "707 - On Thanksgiving Day all over America, families sit down to dinner at the\n"
    "708 - Learning French is trivial: the word for horse is cheval, and everything else\n"
    "709 - You're working under a slight handicap.  You happen to be human.\n"
    "710 - In case of fire, stand in the hall and shout \"Fire!\"\n"
    "711 - <Sanaya> you guys are all sick!  sick sick sick I tell ya ;)\n"
    "712 - vacation, n.:\n"
    "713 - If we men married the women we deserved, we should have a very bad time of it.\n"
    "714 - <gecko> Hmm... I wonder what else seperates Debian from the rest of the\n"
    "715 - Humor in the Court:\n"
    "716 - This planet has -- or rather had -- a problem, which was this:  most of\n"
    "717 - Q:	What's a light-year?\n"
    "718 - What happens to a dream deferred?\n"
    "719 - Disco oil bussing will create a throbbing naugahide pipeline running\n"
    "720 - Disclose classified information only when a NEED TO KNOW exists.\n"
    "721 - Don't get suckered in by the comments -- they can be terribly misleading.\n"
    "722 - panic: kernel trap (ignored)\n"
    "723 - Eating chocolate is like being in love without the aggravation.\n"
    "724 - Time is an illusion, lunchtime doubly so.\n"
    "725 - Mind your own business, Spock.  I'm sick of your halfbreed interference.\n"
    "726 - Give me a Plumber's friend the size of the Pittsburgh dome, and a place\n"
    "727 - Vegetarians beware!  You are what you eat.\n"
    "728 - 	\"Do you believe in intuition?\"\n"
    "729 - Postage will be paid by addressee.\n"
    "730 - If I can have honesty, it's easier to overlook mistakes.\n"
    "731 - Of course a platonic relationship is possible -- but only between\n"
    "732 - I was the best I ever had.\n"
    "733 - If it weren't for the last minute, nothing would ever get done.\n"
    "734 - And the silence came surging softly backwards\n"
    "735 - 	Now, you might ask, \"How do I get one of those complete home tool\n"
    "736 - 	A young married couple had their first child.  Their original pride\n"
    "737 - And all that the Lorax left here in this mess\n"
    "738 - Just when you thought you were winning the rat race, along comes a faster rat!!\n"
    "739 - You will not be elected to public office this year.\n"
    "740 - For if there is a sin against life, it consists perhaps not so much in\n"
    "741 - 	Several students were asked to prove that all odd integers are prime.\n"
    "742 - Q:	How many mathematicians does it take to screw in a light bulb?\n"
    "743 - Al didn't smile for forty years.  You've got to admire a man like that.\n"
    "744 - Zippy's brain cells are straining to bridge synapses ...\n"
    "745 - The more a man is imbued with the ordered regularity of all events, the firmer\n"
    "746 - Girls are better looking in snowstorms.\n"
    "747 - Sturgeon's Law:\n"
    "748 - A mother mouse was taking her large brood for a stroll across the kitchen\n"
    "749 - This restaurant was advertising breakfast any time. So I ordered\n"
    "750 - If I felt any more SOPHISTICATED I would DIE of EMBARRASSMENT!\n"
    "751 - gurmlish, n.:\n"
    "752 - Though I respect that a lot\n"
    "753 - Look ere ye leap.\n"
    "754 - Veni, Vidi, VISA:\n"
    "755 - The ultimate game show will be the one where somebody gets killed at the end.\n"
    "756 - Jesus is my POSTMASTER GENERAL ...\n"
    "757 - In specifications, Murphy's Law supersedes Ohm's.\n"
    "758 - Always the dullness of the fool is the whetstone of the wits.\n"
    "759 - Tax and title extra.\n"
    "760 - 	Idaho state law makes it illegal for a man to give his sweetheart\n"
    "761 - Some marriages are made in heaven -- but so are thunder and lightning.\n"
    "762 - Last night the power went out.  Good thing my camera had a flash....\n"
    "763 - In real love you want the other person's good.  In romantic love you\n"
    "764 - \"The ACLU has stood foursquare against the recurring tides of hysteria that\n"
    "765 - The surest way to corrupt a youth is to instruct him to hold in higher\n"
    "766 - \"Today's robots are very primitive, capable of understanding only a few\n"
    "767 - If you live in a country run by committee, be on the committee.\n"
    "768 - \"If the King's English was good enough for Jesus, it's good enough for me!\"\n"
    "769 - \"It's ten o'clock... Do you know where your AI programs are?\"  -- Peter Oakley\n"
    "770 - Repartee is something we think of twenty-four hours too late.\n"
    "771 - UNIX is many things to many people, but it's never been everything to anybody.\n"
    "772 - One evening Mr. Rudolph Block, of New York, found himself seated at dinner\n"
    "773 - Thus spake the master programmer:\n"
    "774 - At some point, bits have to go into packets and routers need to make\n"
    "775 - Wonderful day.  Your hangover just makes it seem terrible.\n"
    "776 - Intel engineering seem to have misheard Intel marketing strategy. The phrase\n"
    "777 - Shift to the left,\n"
    "778 - ===  ALL CSH USERS PLEASE NOTE  ========================\n"
    "779 - Line Printer paper is strongest at the perforations.\n"
    "780 - Only a fool fights in a burning house.\n"
    "781 - Advertising is the rattling of a stick inside a swill bucket.\n"
    "782 - You will not censor me through bug terrorism.\n"
    "783 - There's nothing like the face of a kid eating a Hershey bar.\n"
    "784 - There is an old custom among my people.  When a woman saves a man's\n"
    "785 - There is nothing wrong with writing ... as long as it is done in private\n"
    "786 - We come to bury DOS, not to praise it.\n"
    "787 - Not every question deserves an answer.\n"
    "788 - <Xavvy> is that really knghtbrd?\n"
    "789 - No, I'm not interested in developing a powerful brain.  All I'm after is\n"
    "790 - Computer programmers never die, they just get lost in the processing.\n"
    "791 - \"I'd love to go out with you, but I'm taking punk totem pole carving.\"\n"
    "792 - All [zoos] actually offer to the public in return for the taxes spent\n"
    "793 - A friend in need is a pest indeed.\n"
    "794 - You know it's Monday when you wake up and it's Tuesday.\n"
    "795 - He who loses, wins the race,\n"
    "796 - The Least Perceptive Literary Critic\n"
    "797 - One of the pleasures of reading old letters is the knowledge that they\n"
    "798 - A full belly makes a dull brain.\n"
    "799 - Every cloud has a silver lining; you should have sold it, and bought titanium.\n"
    "800 - \"No problem is so formidable that you can't walk away from it.\"\n"
    "801 - I might have gone to West Point, but I was too proud to speak to a congressman.\n"
    "802 - \"It is better for civilization to be going down the drain than to be\n"
    "803 - Gordon's Law:\n"
    "804 - Climbing onto a bar stool, a piece of string asked for a beer.\n"
    "805 - 	It is a profoundly erroneous truism, repeated by all copy-books and\n"
    "806 - <Reed> It is important to note that the primary reason the Roman Empire\n"
    "807 - Check me if I'm wrong, Sandy, but if I kill all the golfers...\n"
    "808 - QOTD:\n"
    "809 - Even bytes get lonely for a little bit.\n"
    "810 - Limited time offer, call now to ensure prompt delivery.\n"
    "811 - I can read your mind, and you should be ashamed of yourself.\n"
    "812 - The English instinctively admire any man who has no talent and is modest\n"
    "813 - So I'm ugly.  So what?  I never saw anyone hit with his face.\n"
    "814 - \"I'm a mean green mother from outer space\"\n"
    "815 - Satire is what closes Saturday night.\n"
    "816 - Q:	How many bureaucrats does it take to screw in a light bulb?\n"
    "817 - The bland leadeth the bland and they both shall fall into the kitsch.\n"
    "818 - For years a secret shame destroyed my peace--\n"
    "819 - To be is to do.\n"
    "820 - Bagbiter:\n"
    "821 - EARL GREY PROFILES\n"
    "822 - The trouble with eating Italian food is that five or six days later\n"
    "823 - Delta: A real man lands where he wants to.   -- David Letterman\n"
    "824 - When your memory goes, forget it!\n"
    "825 - Q:	What do little WASPs want to be when they grow up?\n"
    "826 - According to Kentucky state law, every person must take a bath at least\n"
    "827 - I worked in a health food store once.  A guy came in and asked me,\n"
    "828 - QOTD:\n"
    "829 - No committee could ever come up with anything as revolutionary as a camel --\n"
    "830 - One picture is worth 128K words.\n"
    "831 - Documentation is like sex: when it is good, it is very, very good; and\n"
    "832 - Besides, I think Slackware sounds better than 'Microsoft,' don't you?\n"
    "833 - Waving away a cloud of smoke, I look up, and am blinded by a bright, white\n"
    "834 - FORTUNE'S FUN FACTS TO KNOW AND TELL:		#37\n"
    "835 - The heart has its reasons which reason knows nothing of.\n"
    "836 - It's clever, but is it art?\n"
    "837 - Techical solutions are not a matter of voting. Two legislations in the US\n"
    "838 - vuja de:\n"
    "839 - Every morning in Africa, a gazelle wakes up.  It knows it must run faster\n"
    "840 - Not one hundred percent efficient, of course ... but nothing ever is.\n"
    "841 - I love you, not only for what you are, but for what I am when I am with you.\n"
    "842 - It was one time too many\n"
    "843 - Life is difficult because it is non-linear.\n"
    "844 - A diplomatic husband said to his wife, \"How do you expect me to remember\n"
    "845 - If you wish to live wisely, ignore sayings -- including this one.\n"
    "846 - You'll learn something about men and women -- the way they're supposed\n"
    "847 - Today is the first day of the rest of the mess.\n"
    "848 - The advertisement is the most truthful part of a newspaper.\n"
    "849 - Confess your sins to the Lord and you will be forgiven;\n"
    "850 - There's nothing worse for your business than extra Santa Clauses\n"
    "851 - Four thousand different MAGNATES, MOGULS & NABOBS are romping in my\n"
    "852 - If you find a solution and become attached to it, the solution may become\n"
    "853 - Blutarsky's Axiom:\n"
    "854 - I used to be Snow White, but I drifted.\n"
    "855 - <Midgar> From all the sterotypes about Aussies, I figure you guys are\n"
    "856 - This is now.  Later is later.\n"
    "857 - I put the shotgun in an Adidas bag and padded it out with four pairs of\n"
    "858 - He knew the tavernes well in every toun.\n"
    "859 - Atlanta makes it against the law to tie a giraffe to a telephone pole\n"
    "860 - 				FROM THE DESK OF\n"
    "861 - \"Were there no women, men might live like gods.\"\n"
    "862 - Keep your eyes wide open before marriage, half shut afterwards.\n"
    "863 - An economist is a man who would marry Farrah Fawcett-Majors for her money.\n"
    "864 - <robert> i understand there are some reasonable limits to free speech in\n"
    "865 - Death wish, n.:\n"
    "866 - Subject: Bug#42432: debian-policy: Proposal for CTV for Draft for Proof of\n"
    "867 - Life is a grand adventure -- or it is nothing.\n"
    "868 - It's better to be quotable than to be honest.\n"
    "869 - \"He was a modest, good-humored boy.  It was Oxford that made him insufferable.\"\n"
    "870 - You learn to write as if to someone else because NEXT YEAR YOU WILL BE\n"
    "871 - I wouldn't marry her with a ten foot pole.\n"
    "872 - Long computations which yield zero are probably all for naught.\n"
    "873 - Void where prohibited by law.\n"
    "874 - Signs of crime: screaming or cries for help.\n"
    "875 - Documentation is the castor oil of programming.\n"
    "876 - <Knghtbrd> QF is going to get zipfile support today\n"
    "877 - A friend of mine has a barcode on his arm.\n"
    "878 - From the cradle to the coffin underwear comes first.\n"
    "879 - People in general do not willingly read if they have anything else to\n"
    "880 - Yevtushenko has... an ego that can crack crystal at a distance of twenty feet.\n"
    "881 - <Stealth> How do I bind a computer to an NIS server?\n"
    "882 - > The day people think linux would be better served by somebody else (FSF\n"
    "883 - System going down at 5 this afternoon to install scheduler bug.\n"
    "884 - IOT trap -- core dumped\n"
    "885 - Pete:	Waiter, this meat is bad.\n"
    "886 - \"Necessity is the mother of invention\" is a silly proverb.  \"Necessity\n"
    "887 - Overdrawn?  But I still have checks left!\n"
    "888 - The one sure way to make a lazy man look respectable is to put a fishing\n"
    "889 - <Mercury> alexsh: Be /VERY/ cairful, you could, if your unlucky, fry your\n"
    "890 - Many people are unenthusiastic about their work.\n"
    "891 - Ethnomagnetism:\n"
    "892 - Only those who leisurely approach that which the masses are busy about\n"
    "893 - The wages of sin are unreported.\n"
    "894 - Overboarding:\n"
    "895 - I'm a lucky guy, and I'm happy to be with the Yankees.  And I want to\n"
    "896 - Somewhere in suburban Honolulu, an unemployed bellhop is whipping up a\n"
    "897 - Truth is free, but information costs.\n"
    "898 - The difference between this place and yogurt is that yogurt has a live culture.\n"
    "899 - Don't quit now, we might just as well lock the door and throw away the key.\n"
    "900 - The real problem with hunting elephants is carrying the decoys.\n"
    "901 - Actual war is a very messy business.  Very, very messy business.\n"
    "902 - Nitwit ideas are for emergencies.  You use them when you've got nothing\n"
    "903 - Griffin's Thought:\n"
    "904 - Generally speaking, the Way of the warrior is resolute acceptance of death.\n"
    "905 - Anybody want a binary telemetry frame editor written in Perl?\n"
    "906 - Q:	What do you call a boomerang that doesn't come back?\n"
    "907 - Florence Flask was ... dressing for the opera when she turned to her\n"
    "908 - On the Internet, nobody knows you're a dog.\n"
    "909 - Linus Torvalds:\n"
    "910 - Ladles and Jellyspoons!\n"
    "911 -   Evil isn't all bad.\n"
    "912 - One of the things I routinely tell people is that if it's in the news, don't\n"
    "913 - Our houseplants have a good sense of humous.\n"
    "914 - Do not think by infection, catching an opinion like a cold.\n"
    "915 - Xerox never comes up with anything original.\n"
    "916 - Little known fact about Middle Earth: The Hobbits had a very sophisticated\n"
    "917 - Being frustrated is disagreeable, but the real disasters in life begin\n"
    "918 - To downgrade the human mind is bad theology.\n"
    "919 - A sharper perspective on this matter is particularly important to feminist\n"
    "920 - \"The pyramid is opening!\"\n"
    "921 - A furore Normanorum libera nos, O Domine!\n"
    "922 - A modem is a baudy house.\n"
    "923 - He is a man capable of turning any colour into grey.\n"
    "924 - <Overfiend_> Overfiend's First Law of Package Quality: If the\n"
    "925 - In his book, Mr. DePree tells the story of how designer George Nelson urged\n"
    "926 - One day this guy is finally fed up with his middle-class existence and\n"
    "927 - Hark, the Herald Tribune sings,\n"
    "928 - \"We learn from history that we learn nothing from history.\"\n"
    "929 - \"I hate the itching.  But I don't mind the swelling.\"\n"
    "930 - \"If that man in the PTL is such a healer, why can't he make his wife's\n"
    "931 - Never accept an invitation from a stranger unless he gives you candy.\n"
    "932 - Life is a yo-yo, and mankind ties knots in the string.\n"
    "933 - brain-damaged, generalization of \"Honeywell Brain Damage\" (HBD), a\n"
    "934 - Law of Selective Gravity:\n"
    "935 - \"It is hard to overstate the debt that we owe to men and women of genius.\"\n"
    "936 - Scenery is here, wish you were beautiful.\n"
    "937 - And he climbed with the lad up the Eiffelberg Tower.  \"This,\" cried the Mayor,\n"
    "938 - It's hard to tune heavily tuned code.  :-)\n"
    "939 - <Knghtbrd> NOTE THAT THE ABOVE IS JUST AN OPINION AND SHOULD NOT BE\n"
    "940 - Never trust an operating system you don't have sources for. ;-)\n"
    "941 - Just machines to make big decisions,\n"
    "942 - An exotic journey in downtown Newark is in your future.\n"
    "943 - ----==-- _                     / /  \\n"
    "944 - Shall we make a new rule of life from tonight: always to try to be a\n"
    "945 - Ignorance must certainly be bliss or there wouldn't be so many people\n"
    "946 - The grass is always greener on the other side of your sunglasses.\n"
    "947 - \"No matter where you go, there you are...\"\n"
    "948 - 	Once there lived a village of creatures along the bottom of a great\n"
    "949 - Either I'm dead or my watch has stopped.\n"
    "950 - As many of you know, I am taking a class here at UNC on Personality.\n"
    "951 - April 1\n"
    "952 - Do not underestimate the value of print statements for debugging.\n"
    "953 - I've given up reading books; I find it takes my mind off myself.\n"
    "954 - Laugh and the world laughs with you, snore and you sleep alone.\n"
    "955 - It is an important and popular fact that things are not always what\n"
    "956 - quit   When the quit statement is read, the  bc  processor\n"
    "957 - QOTD:\n"
    "958 - If you are going to walk on thin ice, you may as well dance.\n"
    "959 - You can no more win a war than you can win an earthquake.\n"
    "960 - Government [is] an illusion the governed should not encourage.\n"
    "961 - \"I'd crawl over an acre of 'Visual This++' and 'Integrated Development\n"
    "962 - Be assured that a walk through the ocean of most Souls would scarcely\n"
    "963 - The solution to a problem changes the nature of the problem.\n"
    "964 - Beelzebug, n.:\n"
    "965 - Brisk talkers are usually slow thinkers.  There is, indeed, no wild beast\n"
    "966 - A man who keeps stealing mopeds is an obvious cycle-path.\n"
    "967 - Women complain about sex more than men.  Their gripes fall into two\n"
    "968 - While you recently had your problems on the run, they've regrouped and\n"
    "969 - Witch!  Witch!  They'll burn ya!\n"
    "970 - Money is better than poverty, if only for financial reasons.\n"
    "971 - Let the machine do the dirty work.\n"
    "972 - It's not hard to admit errors that are [only] cosmetically wrong.\n"
    "973 - There are two ways of disliking art.   One is to dislike it.  The other is\n"
    "974 - Do not allow this language (Ada) in its present state to be used in\n"
    "975 - The Great Movie Posters:\n"
    "976 - Lowery's Law:\n"
    "977 - We may not be able to persuade Hindus that Jesus and not Vishnu should\n"
    "978 - In general, they do what you want, unless you want consistency.\n"
    "979 - The Worst Musical Trio\n"
    "980 - QOTD:\n"
    "981 - I'm not available for comment..\n"
    "982 - You don't have to be nice to people on the way up if you're not planning on\n"
    "983 - \"It's not just a computer -- it's your ass.\"\n"
    "984 - Change the Social Contract?  BWAHAHAHAHAHAHAHAHAHAHAHA.\n"
    "985 - Die, v.:\n"
    "986 - Keep your boss's boss off your boss's back.\n"
    "987 - A man is crawling through the Sahara desert when he is approached by another\n"
    "988 - It is much easier to be critical than to be correct.\n"
    "989 - If just one piece of mail gets lost, well, they'll just think they forgot\n"
    "990 - I'm QUIETLY reading the latest issue of \"BOWLING WORLD\" while my wife\n"
    "991 - The only way to get rid of a temptation is to yield to it.\n"
    "992 - A man who fishes for marlin in ponds\n"
    "993 - Success is relative: It is what we can make of the mess we have made of things.\n"
    "994 - \"Those who do not do politics will be done in by politics.\"\n"
    "995 - Everyone is more or less mad on one point.\n"
    "996 - Yes, but every time I try to see things your way, I get a headache.\n"
    "997 - Biz is better.\n"
    "998 - \"Old age and treachery will beat youth and skill every time.\"\n"
    "999 - A citizen of America will cross the ocean to fight for democracy, but\n" "1000 - ...I would go so far as to suggest that, were it not for our ego and\n" LASTLINE "\n";

int main(int argc UNUSED, char **argv UNUSED)
{
	int ret = 0;
	GSM_CutLines lines;

	InitLines(&lines);

	SplitLines(input, sizeof(input), &lines, "\n", 1, "", 0, FALSE);

#define COMPARE(num, text) \
	if (strcmp(GetLineString(input, &lines, num), text) != 0) { \
		printf("DIFFERENT[%d]:\n-------\n%s\n-------\n%s\n-------\n", num, GetLineString(input, &lines, num), text); \
		ret = 1; \
		goto end; \
	} \
	if (strlen(GetLineString(input, &lines, num)) != (size_t)GetLineLength(input, &lines, num)) { \
		printf("DIFFERENT LENGTH[%d]: %ld %d\n", num, (long)strlen(GetLineString(input, &lines, num)), GetLineLength(input, &lines, num)); \
		ret = 2; \
		goto end; \
	}

	COMPARE(1, LONGLINE);
	COMPARE(1002, LASTLINE);

end:
	FreeLines(&lines);
	GetLineString(NULL, NULL, 0);

	return ret;
}

/* Editor configuration
 * vim: noexpandtab sw=8 ts=8 sts=8 tw=72:
 */
