#define LISTFORMAT "%-20s : "
#define PRINTSECONDS(num) printf(ngettext("%d second", "%d seconds", num), num);
#define PRINTMINUTES(num) printf(ngettext("%d minute", "%d minutes", num), num);
#define PRINTHOURS(num) printf(ngettext("%d hour", "%d hours", num), num);
#define PRINTDAYS(num) printf(ngettext("%d day", "%d days", num), num);
#define PRINTWEEKS(num) printf(ngettext("%d week", "%d weeks", num), num);
#define PRINTYEARS(num) printf(ngettext("%d year", "%d years", num), num);
