/* (c) 2003-2004 by Marcin Wiacek */

#ifndef __gsm_back_h
#define __gsm_back_h

#ifdef GSM_ENABLE_BACKUP

#include <gammu-backup.h>

#endif
#endif

/* How should editor hadle tabs in this file? Add editor commands here.
 * vim: noexpandtab sw=8 ts=8 sts=8:
 */
